/*
 * wmslib/src/wms/clp.h, part of wmslib (Library functions)
 * Copyright (C) 1994-1996 William Shubert.
 * See "configure.h.in" for more copyright information.
 *
 * Command Line Parse.
 */

#ifndef  _WMS_CLP_X_H_
#define  _WMS_CLP_X_H_  1

#ifndef  _WMS_CLP_H_
#include <wms/clp.h>
#endif


/**********************************************************************
 * Functions
 **********************************************************************/
extern void  clp_rXDefaults(Clp *cltab, Display *dpy, const char *name);


#endif  /* _WMS_CLP_X_H_ */
